@@ -4,6 +4,7 @@
 
 #include "replicator_smm.hpp"
 #include "galera_info.hpp"
+#include "gcs_error.hpp"
 
 #include <gu_abort.h>
 #include <gu_throw.hpp>
@@ -87,6 +88,14 @@ ReplicatorSMM::sst_received(const wsrep_gtid_t& state_id,
         return WSREP_CONN_FAIL;
     }
 
+    if (rcode == -ECANCELED) {
+        // perform a graceful shutdown
+        sst_graceful_shutdown_ = true;
+        sst_received_ = true;
+        sst_cond_.signal();
+        return WSREP_OK;
+    }
+
     assert(rcode <= 0);
     if (rcode) { assert(state_id.seqno == WSREP_SEQNO_UNDEFINED); }
 
@@ -804,12 +813,12 @@ ReplicatorSMM::send_state_request (const StateRequest* const req,
             if (!retry_str(ret))
             {
                 log_error << "Requesting state transfer failed: "
-                          << ret << "(" << strerror(-ret) << ")";
+                          << gcs_state_transfer_error_str(-ret);
             }
             else if (1 == tries)
             {
                 log_info << "Requesting state transfer failed: "
-                         << ret << "(" << strerror(-ret) << "). "
+                         << gcs_state_transfer_error_str(-ret) << ". "
                          << "Will keep retrying every " << sst_retry_sec_
                          << " second(s)";
             }
@@ -861,7 +870,8 @@ ReplicatorSMM::send_state_request (const StateRequest* const req,
         if (!closing_ && state_() > S_CLOSED)
         {
             log_fatal << "State transfer request failed unrecoverably: "
-                      << -ret << " (" << strerror(-ret) << "). Most likely "
+                      << gcs_state_transfer_error_str(-ret)
+                      << ". Most likely "
                       << "it is due to inability to communicate with the "
                       << "cluster primary component. Restart required.";
             abort();
@@ -928,6 +938,19 @@ ReplicatorSMM::request_state_transfer (void* recv_ctx,
             while (false == sst_received_) sst_lock.wait(sst_cond_);
         }
 
+        if (sst_graceful_shutdown_) {
+            log_warn  << "State transfer interrupted, shutting down gracefully:"
+                      << "\n\t sst_uuid = " << sst_uuid_
+                      << "\n\t sst_seqno = " << sst_seqno_
+                      << "\n\t group_uuid = " << group_uuid
+                      << "\n\t safe_to_bootstrap_ = " << safe_to_bootstrap_
+                      << "\n\t cc_seqno = " << cc_seqno;
+
+            st_.restore_saved_state();
+
+            abort();
+        }
+
         if (sst_uuid_ != group_uuid)
         {
             log_fatal << "Application received wrong state: "
@@ -1399,9 +1422,9 @@ void ReplicatorSMM::ist_trx(const TrxHandleSlavePtr& ts, bool must_apply,
     }
 }
 
-void ReplicatorSMM::ist_end(int error)
+void ReplicatorSMM::ist_end(const ist::Result& result)
 {
-    ist_event_queue_.eof(error);
+    ist_event_queue_.eof(result);
 }
 
 void galera::ReplicatorSMM::process_ist_conf_change(const gcs_act_cchange& conf)
