@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2020-2024 Codership Oy <info@codership.com>
+// Copyright (C) 2020-2025 Codership Oy <info@codership.com>
 //
 
 #define GU_ASIO_IMPL
@@ -68,7 +68,7 @@ void gu::AsioStreamReact::open(const gu::URI& uri) try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value())
+    gu_throw_system_error(e.code().value())
         << "error opening stream socket " << uri;
 }
 
@@ -78,11 +78,19 @@ bool gu::AsioStreamReact::is_open() const try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value())
+    gu_throw_system_error(e.code().value())
         << "error checking if socket is open ";
     return false;
 }
 
+void gu::AsioStreamReact::shutdown()
+{
+    if (not (in_progress_ & shutdown_in_progress) && engine_)
+    {
+        engine_->shutdown();
+        in_progress_ |= shutdown_in_progress;
+    }
+}
 
 void gu::AsioStreamReact::close() try
 {
@@ -91,6 +99,7 @@ void gu::AsioStreamReact::close() try
     {
         GU_ASIO_DEBUG(debug_print() << "Socket not open on close");
     }
+
     socket_.close();
 }
 // Catch all the possible exceptions here, not only asio ones.
@@ -105,7 +114,7 @@ void gu::AsioStreamReact::bind(const gu::AsioIpAddress& addr) try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error in binding";
+    gu_throw_system_error(e.code().value()) << "error in binding";
 }
 
 void gu::AsioStreamReact::async_connect(
@@ -127,7 +136,7 @@ void gu::AsioStreamReact::async_connect(
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error connecting ";
+    gu_throw_system_error(e.code().value()) << "error connecting ";
 }
 
 
@@ -149,7 +158,7 @@ void gu::AsioStreamReact::async_write(
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "Async write failed '"
+    gu_throw_system_error(e.code().value()) << "Async write failed '"
                                      << e.what();
 }
 
@@ -169,7 +178,7 @@ void gu::AsioStreamReact::async_read(
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "Async read failed '"
+    gu_throw_system_error(e.code().value()) << "Async read failed '"
                                      << e.what();
 }
 
@@ -178,7 +187,7 @@ static void throw_sync_op_error(const gu::AsioStreamEngine& engine,
 {
     auto last_error(engine.last_error());
     if (last_error.is_system())
-        gu_throw_error(last_error.value()) << prefix
+        gu_throw_system_error(last_error.value()) << prefix
                                            << ": " << last_error.message();
     else
         gu_throw_error(EPROTO) << prefix
@@ -198,6 +207,8 @@ void gu::AsioStreamReact::connect(const gu::URI& uri) try
     socket_.connect(resolve_result->endpoint());
     connected_ = true;
     prepare_engine(false);
+    assign_addresses();
+
     auto result(engine_->client_handshake());
     switch (result)
     {
@@ -215,7 +226,7 @@ void gu::AsioStreamReact::connect(const gu::URI& uri) try
 }
 catch (asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "Failed to connect '"
+    gu_throw_system_error(e.code().value()) << "Failed to connect '"
                                      << uri << "': " << e.what();
 }
 
@@ -242,7 +253,7 @@ size_t gu::AsioStreamReact::write(const AsioConstBuffer& buf) try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "Failed to write: " << e.what();
+    gu_throw_system_error(e.code().value()) << "Failed to write: " << e.what();
 }
 
 size_t gu::AsioStreamReact::read(const AsioMutableBuffer& buf) try
@@ -277,7 +288,7 @@ size_t gu::AsioStreamReact::read(const AsioMutableBuffer& buf) try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "Failed to read: " << e.what();
+    gu_throw_system_error(e.code().value()) << "Failed to read: " << e.what();
 }
 
 std::string gu::AsioStreamReact::local_addr() const
@@ -297,7 +308,7 @@ void gu::AsioStreamReact::set_receive_buffer_size(size_t size) try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error setting receive buffer size";
+    gu_throw_system_error(e.code().value()) << "error setting receive buffer size";
 }
 
 size_t gu::AsioStreamReact::get_receive_buffer_size() try
@@ -306,7 +317,7 @@ size_t gu::AsioStreamReact::get_receive_buffer_size() try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error getting receive buffer size ";
+    gu_throw_system_error(e.code().value()) << "error getting receive buffer size ";
 }
 
 void gu::AsioStreamReact::set_send_buffer_size(size_t size) try
@@ -316,7 +327,7 @@ void gu::AsioStreamReact::set_send_buffer_size(size_t size) try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error setting send buffer size";
+    gu_throw_system_error(e.code().value()) << "error setting send buffer size";
 }
 
 size_t gu::AsioStreamReact::get_send_buffer_size() try
@@ -325,7 +336,7 @@ size_t gu::AsioStreamReact::get_send_buffer_size() try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error getting send buffer size";
+    gu_throw_system_error(e.code().value()) << "error getting send buffer size";
 }
 
 struct tcp_info gu::AsioStreamReact::get_tcp_info() try
@@ -334,7 +345,7 @@ struct tcp_info gu::AsioStreamReact::get_tcp_info() try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error getting TCP info";
+    gu_throw_system_error(e.code().value()) << "error getting TCP info";
 }
 
 
@@ -389,6 +400,7 @@ void gu::AsioStreamReact::connect_handler(
     set_socket_options(socket_);
     prepare_engine(true);
     assign_addresses();
+
     GU_ASIO_DEBUG(debug_print()
                   << " AsioStreamReact::connect_handler: init handshake");
     auto result(engine_->client_handshake());
@@ -683,6 +695,7 @@ void gu::AsioStreamReact::assign_addresses()
         engine_->scheme(),
         ::escape_addr(socket_.remote_endpoint().address()),
         gu::to_string(socket_.remote_endpoint().port()));
+    engine_->update_address_info(local_addr_, remote_addr_);
 }
 
 void gu::AsioStreamReact::prepare_engine(bool non_blocking)
@@ -832,15 +845,6 @@ void gu::AsioStreamReact::set_non_blocking(bool val)
     }
 }
 
-void gu::AsioStreamReact::shutdown()
-{
-    if (not (in_progress_ & shutdown_in_progress) && engine_)
-    {
-        engine_->shutdown();
-        in_progress_ |= shutdown_in_progress;
-    }
-}
-
 std::string gu::AsioStreamReact::debug_print() const
 {
     std::ostringstream oss;
@@ -871,7 +875,12 @@ void gu::AsioAcceptorReact::open(const gu::URI& uri) try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "Failed to open acceptor: " << e.what();
+    gu_throw_system_error(e.code().value()) << "Failed to open acceptor: " << e.what();
+}
+
+bool gu::AsioAcceptorReact::is_open() const
+{
+    return acceptor_.is_open();
 }
 
 
@@ -891,7 +900,7 @@ void gu::AsioAcceptorReact::listen(const gu::URI& uri) try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "Failed to listen: " << e.what();
+    gu_throw_system_error(e.code().value()) << "Failed to listen: " << e.what();
 }
 
 void gu::AsioAcceptorReact::close() try
@@ -904,7 +913,7 @@ void gu::AsioAcceptorReact::close() try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "Failed to close acceptor: "
+    gu_throw_system_error(e.code().value()) << "Failed to close acceptor: "
                                      << e.what();
 }
 
@@ -925,7 +934,7 @@ void gu::AsioAcceptorReact::async_accept(
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "Failed to accept: " << e.what();
+    gu_throw_system_error(e.code().value()) << "Failed to accept: " << e.what();
 }
 
 
@@ -965,7 +974,7 @@ std::shared_ptr<gu::AsioSocket> gu::AsioAcceptorReact::accept() try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "Failed to accept: " << e.what();
+    gu_throw_system_error(e.code().value()) << "Failed to accept: " << e.what();
 }
 
 std::string gu::AsioAcceptorReact::listen_addr() const try
@@ -977,7 +986,7 @@ std::string gu::AsioAcceptorReact::listen_addr() const try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value())
+    gu_throw_system_error(e.code().value())
         << "failed to read listen addr "
         << "', asio error '" << e.what() << "'";
 }
@@ -988,7 +997,7 @@ unsigned short gu::AsioAcceptorReact::listen_port() const try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value())
+    gu_throw_system_error(e.code().value())
         << "failed to read listen port "
         << "', asio error '" << e.what() << "'";
 }
@@ -1000,7 +1009,7 @@ void gu::AsioAcceptorReact::set_receive_buffer_size(size_t size) try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error setting receive buffer size";
+    gu_throw_system_error(e.code().value()) << "error setting receive buffer size";
 }
 
 
@@ -1010,7 +1019,7 @@ size_t gu::AsioAcceptorReact::get_receive_buffer_size() try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error getting receive buffer size";
+    gu_throw_system_error(e.code().value()) << "error getting receive buffer size";
     return 0;
 }
 
@@ -1021,7 +1030,7 @@ void gu::AsioAcceptorReact::set_send_buffer_size(size_t size) try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error setting send buffer size";
+    gu_throw_system_error(e.code().value()) << "error setting send buffer size";
 }
 
 size_t gu::AsioAcceptorReact::get_send_buffer_size() try
@@ -1030,7 +1039,7 @@ size_t gu::AsioAcceptorReact::get_send_buffer_size() try
 }
 catch (const asio::system_error& e)
 {
-    gu_throw_error(e.code().value()) << "error getting send buffer size";
+    gu_throw_system_error(e.code().value()) << "error getting send buffer size";
     return 0;
 }
 
