@@ -8,6 +8,7 @@
 #ifndef __gcache_tests_hpp__
 #define __gcache_tests_hpp__
 
+#include "gcache_enc_test.hpp"
 #include "gcache_mem_test.hpp"
 #include "gcache_rb_test.hpp"
 #include "gcache_page_test.hpp"
@@ -21,6 +22,7 @@ typedef Suite *(*suite_creator_t)(void);
 
 static suite_creator_t suites[] =
 {
+    gcache_enc_suite,
     gcache_mem_suite,
     gcache_rb_suite,
     gcache_page_suite,
