@@ -208,8 +208,8 @@ static inline long gcs_send (gcs_conn_t*    const conn,
                              gcs_act_type_t const act_type,
                              bool           const scheduled)
 {
-    struct gu_buf const buf = { act, static_cast<ssize_t>(act_size) };
-    return gcs_sendv (conn, &buf, act_size, act_type, scheduled, false);
+    struct gu_buf const buf[1] = { act, static_cast<ssize_t>(act_size) };
+    return gcs_sendv (conn, &(buf[0]), act_size, act_type, scheduled, false);
 }
 
 /*!*/
@@ -251,8 +251,8 @@ static inline long gcs_repl (gcs_conn_t*        const conn,
                              struct gcs_action* const action,
                              bool               const scheduled)
 {
-    struct gu_buf const buf = { action->buf, action->size };
-    return gcs_replv (conn, &buf, action, scheduled, nullptr);
+    struct gu_buf const buf[1] = { action->buf, action->size };
+    return gcs_replv (conn, &(buf[0]), action, scheduled, nullptr);
 }
 
 /*! @brief Receives an action from group.
